# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.common import base
from cloudkittyclient import exc


class HashmapManager(base.BaseManager):
    """Class used to manage the Hashmap rating module"""

    url = '/v1/rating/module_config/hashmap/{endpoint}/{resource_id}'

    def get_mapping_types(self, **kwargs):
        """Returns a list of all available mapping types."""
        url = self.get_url('types', kwargs)
        return self.api_client.get(url).json()

    def get_service(self, **kwargs):
        """Returns the service corresponding to the provided ID.

        If no ID is provided, returns a list of all hashmap services.

        :param service_id: ID of the service
        :type service_id: str
        """
        if kwargs.get('service_id'):
            kwargs['resource_id'] = kwargs['service_id']
        url = self.get_url('services', kwargs)
        return self.api_client.get(url).json()

    def create_service(self, **kwargs):
        """Creates a hashmap service.

        :param name: Name of the service
        :type name: str
        """
        if not kwargs.get('name'):
            raise exc.ArgumentRequired("Argument 'service_name' is mandatory.")
        url = self.get_url('services', kwargs)
        body = dict(name=kwargs['name'])
        return self.api_client.post(url, json=body).json()

    def delete_service(self, **kwargs):
        """Deletes a hashmap service

        :param service_id: ID of the service to delete
        :type service_id: uuid
        """
        if not kwargs.get('service_id'):
            raise exc.ArgumentRequired("Argument 'service_id' is mandatory.")
        url = self.get_url('services', kwargs)
        body = dict(service_id=kwargs['service_id'])
        self.api_client.delete(url, json=body)

    def get_field(self, **kwargs):
        """Returns a hashmap field.

        Either service_id or field_id must be specified. If service_id is
        provided, all fields of the given service are returned. If field_id
        is specified, only this field is returned.

        :param service_id: ID of the service of which you want fields
        :type service_id: str
        :param field_id: ID of the field you want
        :type field_id: str
        """
        if not kwargs.get('service_id') and not kwargs.get('field_id'):
            raise exc.ArgumentRequired("Either 'service_id' or 'field_id' "
                                       "must be specified.")
        elif kwargs.get('service_id') and kwargs.get('field_id'):
            raise exc.InvalidArgumentError(
                "You can't specify both 'service_id' and 'field_id'")
        elif kwargs.get('field_id'):
            kwargs['resource_id'] = kwargs['field_id']
            kwargs.pop('service_id', None)
        else:
            kwargs.pop('resource_id', None)
        authorized_args = ['service_id']
        url = self.get_url('fields', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def create_field(self, **kwargs):
        """Creates a hashmap field.

        :param name: Field name
        :type name: str
        :param service_id: ID of the service the field belongs to
        :type service_id: uuid
        """
        if not kwargs.get('name'):
            raise exc.ArgumentRequired("'name' argument is required")
        if not kwargs.get('service_id'):
            raise exc.ArgumentRequired("'service_id' argument is required")
        body = dict(name=kwargs['name'], service_id=kwargs['service_id'])
        url = self.get_url('fields', kwargs)
        return self.api_client.post(url, json=body).json()

    def delete_field(self, **kwargs):
        """Deletes the given field.

        :param field_id: ID of the field to delete.
        :type field_id: uuid
        """
        if not kwargs.get('field_id'):
            raise exc.ArgumentRequired("'field_id' argument is required")
        url = self.get_url('fields', kwargs)
        body = dict(field_id=kwargs['field_id'])
        self.api_client.delete(url, json=body)

    def get_mapping(self, **kwargs):
        """Get hashmap mappings.

        If mapping_id is not provided, you need to specify either service_id,
        field_id or group_id.

        :param mapping_id: ID of the mapping
        :type mapping_id: uuid
        :param service_id: ID of the service to filter on
        :type service_id: uuid
        :param group_id: ID of the group to filter on
        :type group_id: uuid
        :param field_id: ID of the field to filter on
        :type field_id: uuid
        :param tenant_id: ID of the tenant to filter on
        :type tenant_id: uuid
        :param filter_tenant: Explicitly filter on given tenant (allows to
                              filter on tenant being None). Defaults to false.
        :type filter_tenant: bool
        :param no_group: Filter on orphaned mappings.
        :type no_group: bool
        """
        if not kwargs.get('mapping_id'):
            if not kwargs.get('service_id') and not kwargs.get('field_id') \
               and not kwargs.get('group_id'):
                raise exc.ArgumentRequired("You must provide either 'field_id'"
                                           ", 'service_id' or 'group_id'.")
            allowed_args = ['service_id', 'group_id', 'field_id', 'tenant_id',
                            'filter_tenant', 'no_group']
        else:
            allowed_args = []
            kwargs['resource_id'] = kwargs['mapping_id']
        url = self.get_url('mappings', kwargs, allowed_args)
        return self.api_client.get(url).json()

    def create_mapping(self, **kwargs):
        """Create a hashmap mapping.

        :param cost: Cost of the mapping
        :type cost: decimal.Decimal
        :param field_id: ID of the field the mapping belongs to
        :type field_id: uuid
        :param service_id: ID of the service the mapping belongs to
        :type service_id: uuid
        :param tenant_id: ID of the tenant the mapping belongs to
        :type tenant_id: uuid
        :param group_id: ID of the group the mapping belongs to
        :type group_id: uuid
        :param type: Type of the mapping (flat or rate)
        :type type: str
        :param value: Value of the mapping
        :type value: str
        """
        if kwargs.get('cost') is None:
            raise exc.ArgumentRequired("'cost' argument is required")
        if not kwargs.get('value'):
            if not kwargs.get('service_id'):
                raise exc.ArgumentRequired(
                    "'service_id' must be specified if no value is provided")
        if kwargs.get('value') and kwargs.get('service_id'):
            raise exc.InvalidArgumentError(
                "You can't specify a value when 'service_id' is specified.")
        if not kwargs.get('service_id') and not kwargs.get('field_id'):
            raise exc.ArgumentRequired("You must specify either 'service_id'"
                                       " or 'field_id'")
        elif kwargs.get('service_id') and kwargs.get('field_id'):
            raise exc.InvalidArgumentError(
                "You can't specify both 'service_id'and 'field_id'")
        body = dict(
            cost=kwargs.get('cost'),
            value=kwargs.get('value'),
            service_id=kwargs.get('service_id'),
            group_id=kwargs.get('group_id'),
            field_id=kwargs.get('field_id'),
            tenant_id=kwargs.get('tenant_id'),
            type=kwargs.get('type') or 'flat',
        )
        url = self.get_url('mappings', kwargs)
        return self.api_client.post(url, json=body).json()

    def delete_mapping(self, **kwargs):
        """Delete a hashmap mapping.

        :param mapping_id: ID of the mapping to delete.
        :type mapping_id: uuid
        """
        if not kwargs.get('mapping_id'):
            raise exc.ArgumentRequired("'mapping_id' argument is required")
        url = self.get_url('mappings', kwargs)
        body = dict(mapping_id=kwargs['mapping_id'])
        self.api_client.delete(url, json=body)

    def update_mapping(self, **kwargs):
        """Update a hashmap mapping.

        :param mapping_id: ID of the mapping to update
        :type mapping_id: uuid
        :param cost: Cost of the mapping
        :type cost: decimal.Decimal
        :param field_id: ID of the field the mapping belongs to
        :type field_id: uuid
        :param service_id: ID of the field the mapping belongs to
        :type service_id: uuid
        :param tenant_id: ID of the field the mapping belongs to
        :type tenant_id: uuid
        :param type: Type of the mapping (flat or rate)
        :type type: str
        :param value: Value of the mapping
        :type value: str
        """
        if not kwargs.get('mapping_id'):
            raise exc.ArgumentRequired("'mapping_id' argument is required")
        mapping = self.get_mapping(**kwargs)
        for key in mapping.keys():
            value = kwargs.get(key, None)
            if value is not None and mapping[key] != value:
                mapping[key] = value
        url = self.get_url('mappings', kwargs)
        self.api_client.put(url, json=mapping)
        return self.get_mapping(**kwargs)

    def get_mapping_group(self, **kwargs):
        """Get the group attached to a mapping.

        :param mapping_id: ID of the mapping to update
        :type mapping_id: uuid
        """
        if not kwargs.get('mapping_id'):
            raise exc.ArgumentRequired("'mapping_id' argument is required")
        kwargs['resource_id'] = 'group'
        allowed_args = ['mapping_id']
        url = self.get_url('mappings', kwargs, allowed_args)
        return self.api_client.get(url).json()

    def get_group(self, **kwargs):
        """Get the hashmap group corresponding to the given ID.

        If group_id is not specified, returns a list of all hashmap groups.

        :param group_id: Group ID
        :type group_id: uuid
        """
        kwargs['resource_id'] = kwargs.get('group_id') or ''
        url = self.get_url('groups', kwargs)
        return self.api_client.get(url).json()

    def create_group(self, **kwargs):
        """Create a hashmap group.

        :param name: Name of the group
        :type name: str
        """
        if not kwargs.get('name'):
            raise exc.ArgumentRequired("'name' argument is required")
        body = dict(name=kwargs['name'])
        url = self.get_url('groups', kwargs)
        return self.api_client.post(url, json=body).json()

    def delete_group(self, **kwargs):
        """Delete a hashmap group.

        :param group_id: ID of the group to delete
        :type group_id: uuid
        :param recursive: Delete mappings recursively
        :type recursive: bool
        """
        if not kwargs.get('group_id'):
            raise exc.ArgumentRequired("'group_id' argument is required")
        body = dict(
            group_id=kwargs['group_id'],
            recursive=kwargs.get('recursive', False))
        url = self.get_url('groups', kwargs)
        self.api_client.delete(url, json=body)

    def get_group_mappings(self, **kwargs):
        """Get the mappings attached to the given group.

        :param group_id: ID of the group
        :type group_id: uuid
        """
        if not kwargs.get('group_id'):
            raise exc.ArgumentRequired("'group_id' argument is required")
        authorized_args = ['group_id']
        kwargs['resource_id'] = 'mappings'
        url = self.get_url('groups', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def get_group_thresholds(self, **kwargs):
        """Get the thresholds attached to the given group.

        :param group_id: ID of the group
        :type group_id: uuid
        """
        if not kwargs.get('group_id'):
            raise exc.ArgumentRequired("'group_id' argument is required")
        authorized_args = ['group_id']
        kwargs['resource_id'] = 'thresholds'
        url = self.get_url('groups', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def get_threshold(self, **kwargs):
        """Get hashmap thresholds.

        If threshold_id is not provided, you need to specify either service_id,
        field_id or group_id.

        :param threshold_id: ID of the threshold
        :type threshold_id: uuid
        :param service_id: ID of the service to filter on
        :type service_id: uuid
        :param group_id: ID of the group to filter on
        :type group_id: uuid
        :param field_id: ID of the field to filter on
        :type field_id: uuid
        :param tenant_id: ID of the tenant to filter on
        :type tenant_id: uuid
        :param filter_tenant: Explicitly filter on given tenant (allows to
                              filter on tenant being None). Defaults to false.
        :type filter_tenant: bool
        :param no_group: Filter on orphaned thresholds.
        :type no_group: bool
        """
        if not kwargs.get('threshold_id'):
            if not kwargs.get('service_id') and not kwargs.get('field_id') \
               and not kwargs.get('group_id'):
                raise exc.ArgumentRequired("You must provide either 'field_id'"
                                           ", 'service_id' or 'group_id'.")
            allowed_args = ['service_id', 'group_id', 'field_id', 'tenant_id',
                            'filter_tenant', 'no_group']
        else:
            allowed_args = []
            kwargs['resource_id'] = kwargs['threshold_id']
        url = self.get_url('thresholds', kwargs, allowed_args)
        return self.api_client.get(url).json()

    def create_threshold(self, **kwargs):
        """Create a hashmap threshold.

        :param cost: Cost of the threshold
        :type cost: decimal.Decimal
        :param field_id: ID of the field the threshold belongs to
        :type field_id: uuid
        :param service_id: ID of the service the threshold belongs to
        :type service_id: uuid
        :param tenant_id: ID of the tenant the threshold belongs to
        :type tenant_id: uuid
        :param group_id: ID of the group the threshold belongs to
        :type group_id: uuid
        :param type: Type of the threshold (flat or rate)
        :type type: str
        :param level: Level of the threshold
        :type level: str
        """
        for arg in ['cost', 'level']:
            if kwargs.get(arg) is None:
                raise exc.ArgumentRequired(
                    "'{}' argument is required".format(arg))
        if not kwargs.get('service_id') and not kwargs.get('field_id'):
            raise exc.ArgumentRequired("You must specify either 'service_id'"
                                       " or 'field_id'")
        body = dict(
            cost=kwargs.get('cost'),
            level=kwargs.get('level'),
            service_id=kwargs.get('service_id'),
            field_id=kwargs.get('field_id'),
            group_id=kwargs.get('group_id'),
            tenant_id=kwargs.get('tenant_id'),
            type=kwargs.get('type') or 'flat',
        )
        url = self.get_url('thresholds', kwargs)
        return self.api_client.post(url, json=body).json()

    def delete_threshold(self, **kwargs):
        """Delete a hashmap threshold.

        :param threshold_id: ID of the threshold to delete.
        :type threshold_id: uuid
        """
        if not kwargs.get('threshold_id'):
            raise exc.ArgumentRequired("'threshold_id' argument is required")
        url = self.get_url('thresholds', kwargs)
        body = dict(threshold_id=kwargs['threshold_id'])
        self.api_client.delete(url, json=body)

    def update_threshold(self, **kwargs):
        """Update a hashmap threshold.

        :param threshold_id: ID of the threshold to update
        :type threshold_id: uuid
        :param cost: Cost of the threshold
        :type cost: decimal.Decimal
        :param field_id: ID of the field the threshold belongs to
        :type field_id: uuid
        :param service_id: ID of the field the threshold belongs to
        :type service_id: uuid
        :param tenant_id: ID of the field the threshold belongs to
        :type tenant_id: uuid
        :param type: Type of the threshold (flat or rate)
        :type type: str
        :param level: Level of the threshold
        :type level: str
        """
        if not kwargs.get('threshold_id'):
            raise exc.ArgumentRequired("'threshold_id' argument is required")
        threshold = self.get_threshold(**kwargs)
        for key in threshold.keys():
            value = kwargs.get(key, None)
            if value is not None and threshold[key] != value:
                threshold[key] = value
        url = self.get_url('thresholds', kwargs)
        self.api_client.put(url, json=threshold)
        return self.get_threshold(**kwargs)

    def get_threshold_group(self, **kwargs):
        """Get the group attached to a threshold.

        :param threshold_id: ID of the threshold to update
        :type threshold_id: uuid
        """
        if not kwargs.get('threshold_id'):
            raise exc.ArgumentRequired("'threshold_id' argument is required")
        kwargs['resource_id'] = 'group'
        allowed_args = ['threshold_id']
        url = self.get_url('thresholds', kwargs, allowed_args)
        return self.api_client.get(url).json()



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.common import base


class SummaryManager(base.BaseManager):

    url = '/v2/summary'

    def get_summary(self, **kwargs):
        """Returns a paginated list of summaries.

        This support filters along with custom grouping.

        :param offset: Index of the first scope that should be returned.
        :type offset: int
        :param limit: Maximal number of scopes to return.
        :type limit: int
        :param filters: Optional dict of filters to select data on.
        :type filters: dict
        :param groupby: Optional list of attributes to group data on.
        :type groupby: str or list of str.
        :param begin: Start of the period to gather data from
        :type begin: datetime.datetime
        :param end: End of the period to gather data from
        :type end: datetime.datetime
        """
        if 'groupby' in kwargs.keys() and isinstance(kwargs['groupby'], list):
            kwargs['groupby'] = ','.join(kwargs['groupby'])

        kwargs['filters'] = ','.join(
            '{}:{}'.format(k, v) for k, v in
            (kwargs.get('filters', None) or {}).items()
        )

        authorized_args = [
            'offset', 'limit', 'filters', 'groupby', 'begin', 'end']

        url = self.get_url(None, kwargs, authorized_args=authorized_args)
        return self.api_client.get(url).json()



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import lister

from cloudkittyclient import exc
from cloudkittyclient import utils


class CliGetDataframes(lister.Lister):
    """List stored dataframes or generate CSV reports.

    Dataframes can be filtered on resource_type and project_id.
    CSV reports can be generated with the 'df-to-csv' formatter.
    A config file may be provided to configure the output of that formatter.
    See documentation for more details.
    """
    columns = [
        ('begin', 'Begin'),
        ('end', 'End'),
        ('tenant_id', 'Project ID'),
        ('resources', 'Resources'),
    ]

    def take_action(self, parsed_args):
        for arg in ['begin', 'end']:
            value = getattr(parsed_args, arg)
            if value is not None:
                try:
                    setattr(parsed_args, arg, utils.iso2dt(value))
                except ValueError:
                    raise exc.InvalidArgumentError(
                        'Invalid timestamp "{}"'.format(value))
        resp = utils.get_client_from_osc(self).storage.get_dataframes(
            **vars(parsed_args)).get('dataframes', [])
        values = utils.list_to_cols(resp, self.columns)
        for value in values:
            for resource in value[3]:
                rating = float(resource['rating'])
                volume = float(resource['volume'])
                if volume > 0:
                    resource['rate_value'] = '{:.4f}'.format(rating / volume)
                else:
                    resource['rate_value'] = ''
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliGetDataframes, self).get_parser(prog_name)
        parser.add_argument('-b', '--begin', type=str, help='Begin timestamp')
        parser.add_argument('-e', '--end', type=str, help='End timestamp')
        parser.add_argument('-p', '--project-id', type=str, dest='tenant_id',
                            help='Id of the tenant to filter on')
        parser.add_argument('-r', '--resource_type', type=str,
                            help='Resource type to filter on')
        return parser



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient import exc
from cloudkittyclient.tests.unit.v2 import base
import datetime


class TestScope(base.BaseAPIEndpointTestCase):

    def test_get_scope(self):
        self.scope.get_scope_state()
        self.api_client.get.assert_called_once_with('/v2/scope')

    def test_get_scope_with_args(self):
        self.scope.get_scope_state(offset=10, limit=10)
        try:
            self.api_client.get.assert_called_once_with(
                '/v2/scope?limit=10&offset=10')
        except AssertionError:
            self.api_client.get.assert_called_once_with(
                '/v2/scope?offset=10&limit=10')

    def test_reset_scope_with_args(self):
        self.scope.reset_scope_state(
            state=datetime.datetime(2019, 5, 7),
            all_scopes=True)
        self.api_client.put.assert_called_once_with(
            '/v2/scope',
            json={
                'state': datetime.datetime(2019, 5, 7),
                'all_scopes': True,
            })

    def test_reset_scope_with_list_args(self):
        self.scope.reset_scope_state(
            state=datetime.datetime(2019, 5, 7),
            scope_id=['id1', 'id2'],
            all_scopes=False)
        self.api_client.put.assert_called_once_with(
            '/v2/scope',
            json={
                'state': datetime.datetime(2019, 5, 7),
                'scope_id': 'id1,id2',
            })

    def test_reset_scope_strips_none_and_false_args(self):
        self.scope.reset_scope_state(
            state=datetime.datetime(2019, 5, 7),
            all_scopes=False,
            scope_key=None,
            scope_id=['id1', 'id2'])
        self.api_client.put.assert_called_once_with(
            '/v2/scope',
            json={
                'state': datetime.datetime(2019, 5, 7),
                'scope_id': 'id1,id2',
            })

    def test_reset_scope_with_no_args_raises_exc(self):
        self.assertRaises(
            exc.ArgumentRequired,
            self.scope.reset_scope_state)

    def test_reset_scope_with_lacking_args_raises_exc(self):
        self.assertRaises(
            exc.ArgumentRequired,
            self.scope.reset_scope_state,
            state=datetime.datetime(2019, 5, 7))

    def test_reset_scope_with_both_args_raises_exc(self):
        self.assertRaises(
            exc.InvalidArgumentError,
            self.scope.reset_scope_state,
            state=datetime.datetime(2019, 5, 7),
            scope_id=['id1', 'id2'],
            all_scopes=True)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import mock

import fixtures
import testtools

from keystoneauth1 import adapter
from keystoneauth1 import session


class BaseTestCase(testtools.TestCase):

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.useFixture(fixtures.FakeLogger())


class FakeRequest(dict):
    """Fake requests.Request object."""

    def json(self):
        return self


class FakeHTTPClient(adapter.Adapter):
    """Keystone HTTP adapter with request methods being mocks"""

    def __init__(self):
        super(FakeHTTPClient, self).__init__(session=session.Session())
        for attr in ('get', 'put', 'post', 'delete'):
            setattr(self, attr, mock.Mock(return_value=FakeRequest()))



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import command
from cliff import lister

from cloudkittyclient import utils


class CliCollectorMappingGet(lister.Lister):
    """(DEPRECATED) Get a service to collector mapping."""

    columns = [
        ('service', 'Service'),
        ('collector', 'Collector'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).collector.get_mapping(
            service=parsed_args.service,
        )
        resp = [resp] if resp.get('mappings') is None else resp['mappings']
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCollectorMappingGet, self).get_parser(prog_name)
        parser.add_argument('service', type=str,
                            help='Name of the service to filter on')
        return parser


class CliCollectorMappingList(lister.Lister):
    """(DEPRECATED) List service to collector mappings."""

    columns = [
        ('service', 'Service'),
        ('collector', 'Collector'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).collector.get_mapping(
            collector=parsed_args.collector)
        resp = [resp] if resp.get('mappings') is None else resp['mappings']
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCollectorMappingList, self).get_parser(prog_name)
        parser.add_argument('--collector', type=str,
                            help='Name of the collector to filter on')
        return parser


class CliCollectorMappingCreate(lister.Lister):
    """(DEPRECATED) Create a service to collector mapping."""

    columns = [
        ('service', 'Service'),
        ('collector', 'Collector'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).collector.create_mapping(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCollectorMappingCreate, self).get_parser(prog_name)
        parser.add_argument('service', type=str, help='Name of the service')
        parser.add_argument('collector', type=str,
                            help='Name of the collector')
        return parser


class CliCollectorMappingDelete(command.Command):
    """(DEPRECATED) Delete a service to collector mapping."""

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).collector.delete_mapping(
            **vars(parsed_args))

    def get_parser(self, prog_name):
        parser = super(CliCollectorMappingDelete, self).get_parser(prog_name)
        parser.add_argument('service', type=str, help='Name of the service')
        return parser


class CliCollectorGetState(lister.Lister):
    """(DEPRECATED) Get the state of a collector."""

    columns = [
        ('name', 'Collector'),
        ('enabled', 'State'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).collector.get_state(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCollectorGetState, self).get_parser(prog_name)
        parser.add_argument('name', type=str, help='Name of the collector')
        return parser


class CliCollectorEnable(lister.Lister):
    """(DEPRECATED) Enable a collector."""

    columns = [
        ('name', 'Collector'),
        ('enabled', 'State'),
    ]

    def take_action(self, parsed_args):
        parsed_args.enabled = True
        resp = utils.get_client_from_osc(self).collector.set_state(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCollectorEnable, self).get_parser(prog_name)
        parser.add_argument('name', type=str, help='Name of the collector')
        return parser


class CliCollectorDisable(CliCollectorEnable):
    """(DEPRECATED) Disable a collector."""

    def take_action(self, parsed_args):
        parsed_args.disabled = True
        resp = utils.get_client_from_osc(self).collector.set_state(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from oslo_log import log

from cloudkittyclient.common import base


LOG = log.getLogger(__name__)


class ReportManager(base.BaseManager):
    """Class used to handle /v1/report endpoint."""
    url = '/v1/report/{endpoint}'

    def get_summary(self, **kwargs):
        """Returns a list of summaries.

        :param begin: Begin timestamp
        :type begin: datetime.datetime
        :param end: End timestamp
        :type end: datetime.datetime
        :param tenant_id: Tenant ID
        :type tenant_id: str
        :param groupby: Fields to group by.
        :type groupby: list
        :param all_tenants: Get summary from all tenants (admin only). Defaults
                            to False.
        :type all_tenants: bool
        """
        authorized_args = [
            'begin', 'end', 'tenant_id', 'service', 'groupby', 'all_tenants']
        if kwargs.get('groupby', None):
            kwargs['groupby'] = ','.join(kwargs['groupby'])
        url = self.get_url('summary', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def get_total(self, **kwargs):
        """Returns the total for the given tenant.

        :param begin: Begin timestamp
        :type begin: datetime.datetime
        :param end: End timestamp
        :type end: datetime.datetime
        :param tenant_id: Tenant ID
        :type tenant_id: str
        :param all_tenants: Get total from all tenants (admin only). Defaults
                            to False.
        :type all_tenants: bool
        """
        LOG.warning('WARNING: /v1/report/total/ endpoint is deprecated, '
                    'please use /v1/report/summary instead.')
        authorized_args = [
            'begin', 'end', 'tenant_id', 'service', 'all_tenants']
        url = self.get_url('total', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def get_tenants(self, **kwargs):
        """Returns a list of tenants.

        :param begin: Begin timestamp
        :type begin: datetime.datetime
        :param end: End timestamp
        :type end: datetime.datetime
        """
        url = self.get_url('tenants', kwargs, ['begin', 'end'])
        return self.api_client.get(url).json()
